@@ -150,6 +150,9 @@ class SettingsSection(Gtk.Box):
         Gtk.Box.__init__(self, orientation=Gtk.Orientation.VERTICAL)
         self.set_spacing(10)
 
+        self.always_show = False
+        self.revealers = []
+
         if title or subtitle:
             header_box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
             header_box.set_spacing(5)
@@ -169,6 +172,7 @@ class SettingsSection(Gtk.Box):
                 header_box.add(sub)
 
         self.frame = Gtk.Frame()
+        self.frame.set_no_show_all(True)
         self.frame.set_shadow_type(Gtk.ShadowType.IN)
         frame_style = self.frame.get_style_context()
         frame_style.add_class("view")
@@ -177,6 +181,7 @@ class SettingsSection(Gtk.Box):
 
         self.box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
         self.frame.add(self.box)
+        self.add(self.frame)
 
         self.need_separator = False
 
@@ -194,8 +199,7 @@ class SettingsSection(Gtk.Box):
         vbox.add(list_box)
         self.box.add(vbox)
 
-        if self.frame.get_parent() is None:
-            self.add(self.frame)
+        self.update_always_show_state()
 
         self.need_separator = True
 
@@ -219,6 +223,11 @@ class SettingsSection(Gtk.Box):
 
         self.need_separator = True
 
+        self.revealers.append(revealer)
+        if not self.always_show:
+            revealer.notify_id = revealer.connect('notify::child-revealed', self.check_reveal_state)
+            self.check_reveal_state()
+
         return revealer
 
     def add_note(self, text):
@@ -229,6 +238,26 @@ class SettingsSection(Gtk.Box):
         self.add(label)
         return label
 
+    def update_always_show_state(self):
+        if self.always_show:
+            return
+
+        self.frame.set_no_show_all(False)
+        self.frame.show_all()
+        self.always_show = True
+
+        for revealer in self.revealers:
+            revealer.disconnect(revealer.notify_id)
+
+    def check_reveal_state(self, *args):
+        for revealer in self.revealers:
+            if revealer.props.child_revealed:
+                self.box.show_all()
+                self.frame.show()
+                return
+
+        self.frame.hide()
+
 class SettingsWidget(Gtk.Box):
     def __init__(self, dep_key=None):
         Gtk.Box.__init__(self)
@@ -400,7 +429,7 @@ class FontButton(SettingsWidget):
     bind_prop = "font-name"
     bind_dir = Gio.SettingsBindFlags.DEFAULT
 
-    def __init__(self, label, size_group=None, dep_key=None, tooltip=""):
+    def __init__(self, label, level=(Gtk.FontChooserLevel.STYLE | Gtk.FontChooserLevel.SIZE), size_group=None, dep_key=None, tooltip=""):
         super(FontButton, self).__init__(dep_key=dep_key)
 
         self.label = SettingsLabel(label)
@@ -416,6 +445,27 @@ class FontButton(SettingsWidget):
         if size_group:
             self.add_to_size_group(size_group)
 
+        if not (level & Gtk.FontChooserLevel.STYLE):
+            # the level mechanism is broken in gtk - choosing a level that doesn't include styles (italics, bold, etc)
+            # results in a list that only has one variant per family, but in many cases the selected font is bold or
+            # italicized (but not consistenly), which is not only confusing, but kind of defeats the purpose. To work
+            # around that, we supply our own filter function that removes the variants correctly. If it ever gets fixed
+            # in gtk, this parsing function can be removed and the level argument added directly to the font button.
+            def filter_func(fam, face):
+                face_text = face.get_face_name().lower()
+                for keyword in ['bold', 'italic', 'oblique']:
+                    if keyword in face_text:
+                        return False
+
+                return True
+
+            self.content_widget.set_filter_func(filter_func)
+
+            # add the style level to avoid the issues above - it's because we're already filtering them out manually
+            level |= Gtk.FontChooserLevel.STYLE
+
+        self.content_widget.set_level(level)
+
 class Range(SettingsWidget):
     bind_prop = "value"
     bind_dir = Gio.SettingsBindFlags.GET | Gio.SettingsBindFlags.NO_SENSITIVITY
@@ -498,8 +548,17 @@ class Range(SettingsWidget):
         self.content_widget.connect("scroll-event", self.on_scroll_event)
         self.content_widget.connect("value-changed", self.apply_later)
 
+        if (not log) and self.step % 1 == 0:
+            self.content_widget.connect("change-value", self.round_value_to_step)
+
         self.set_tooltip_text(tooltip)
 
+    def round_value_to_step(self, widget, scroll, value, data=None):
+        if value % self.step != 0:
+            widget.set_value(round(value / self.step) * self.step)
+            return True
+        return False
+
     def apply_later(self, *args):
         def apply(self):
             if self.log:
@@ -537,10 +596,11 @@ class Range(SettingsWidget):
 class ComboBox(SettingsWidget):
     bind_dir = None
 
-    def __init__(self, label, options=[], valtype=None, size_group=None, dep_key=None, tooltip=""):
+    def __init__(self, label, options=[], valtype=None, separator=None, size_group=None, dep_key=None, tooltip=""):
         super(ComboBox, self).__init__(dep_key=dep_key)
 
         self.valtype = valtype
+        self.separator = separator
         self.option_map = {}
 
         self.label = SettingsLabel(label)
@@ -556,6 +616,9 @@ class ComboBox(SettingsWidget):
 
         self.set_options(options)
 
+        if separator:
+            self.content_widget.set_row_separator_func(self.is_separator_row)
+
         self.set_tooltip_text(tooltip)
 
         if size_group:
@@ -591,6 +654,12 @@ class ComboBox(SettingsWidget):
         self.content_widget.set_model(self.model)
         self.content_widget.set_id_column(0)
 
+    def is_separator_row(self, model, tree_iter):
+        if model[tree_iter][0] == self.separator:
+            return True
+        else:
+            return False
+
 class ColorChooser(SettingsWidget):
     bind_dir = None
 
@@ -661,7 +730,7 @@ class IconChooser(SettingsWidget):
     bind_prop = "icon"
     bind_dir = Gio.SettingsBindFlags.DEFAULT
 
-    def __init__(self, label, default_icon=None, custom=[], expand_width=False, size_group=None, dep_key=None, tooltip=""):
+    def __init__(self, label, default_icon=None, icon_categories=[], default_category=None, expand_width=False, size_group=None, dep_key=None, tooltip=""):
         super(IconChooser, self).__init__(dep_key=dep_key)
 
         self.label = SettingsLabel(label)
@@ -673,8 +742,11 @@ class IconChooser(SettingsWidget):
         if default_icon:
             dialog.set_default_icon(default_icon)
 
-        for item in custom:
-            dialog.add_custom_category(item['name'], item['icons'])
+        for category in icon_categories:
+            dialog.add_custom_category(category['name'], category['icons'])
+
+        if default_category is not None:
+            self.content_widget.set_default_category(default_category)
 
         self.pack_start(self.label, False, False, 0)
         self.pack_end(self.content_widget, expand_width, expand_width, 0)
